@@ -1,3 +1,10 @@
+#ifndef __PBDBASE_BASE_STATS__
+#define __PBDBASE_BASE_STATS__
+
+
 // variance.f90
 void pdclmn_(double *x, int *descx, double *mn);
 void pdclvar_(double *x, int *descx, double *var);
+
+
+#endif
